# Generated by roxygen2: do not edit by hand

export(auc)
export(block_tidydesign)
export(calc_deriv)
export(centroid)
export(centroid_both)
export(centroid_x)
export(centroid_y)
export(doubling_time)
export(extr_val)
export(find_local_extrema)
export(find_threshold_crosses)
export(first_above)
export(first_below)
export(first_maxima)
export(first_minima)
export(first_peak)
export(from_excel)
export(gc_smooth.spline)
export(import_blockdesigns)
export(import_blockmeasures)
export(lag_time)
export(make_design)
export(make_designpattern)
export(make_example)
export(make_tidydesign)
export(makemethod_train_smooth_data)
export(max_gc)
export(mdp)
export(merge_dfs)
export(min_gc)
export(moving_average)
export(moving_median)
export(paste_blocks)
export(predict_interpolation)
export(print_df)
export(read_blocks)
export(read_tidys)
export(read_wides)
export(separate_tidy)
export(smooth_data)
export(solve_linear)
export(to_excel)
export(train_smooth_data)
export(trans_block_to_wide)
export(trans_wide_to_tidy)
export(uninterleave)
export(which_max_gc)
export(which_min_gc)
export(write_blocks)
importFrom(rlang,.data)
